#!/bin/sh
source ./speedtest.conf
source ./router.conf

source ./firebase.conf
RESP_TOKEN=$(/usr/bin/curl -s "$TOKEN_URL" \
        -H 'Content-Type: application/json' \
        --data-binary '{"email":"'$FB_EMAIL'","password":"'$FB_PASSWORD'","returnSecureToken":true}')
TOKEN=$(echo $RESP_TOKEN | /usr/bin/jsonfilter -e '$.idToken')

speedcheck_prepare()
{
        local server="$1"
        local url1 url2 output answer port checksum

        url1="http://$server/cgi-bin-speedometer.sh?TOKEN=${TOKEN}&BITRATE=1&LINEID=foo&MTYPE=iperf3"
        url2="http://$server/cgi-bin/cgi-bin-speedometer.sh?TOKEN=${TOKEN}&BITRATE=1&LINEID=foo&MTYPE=iperf3"

        # e.g. OK 12345 sha256sum
        output="$( wget -T 10 -qO - "$url2" )"        # url1 = openwrt | url2 = debian

        set -f; set +f -- $output
        answer=$1       # e.g. OK
        port=$2         # e.g. 12345
        checksum=$3     # e.g. 64byte-hex-sha256sum

        test "$answer" = 'OK' || {
                logger -s "error: answer != ok - out: '$output'"
                return 64
        }

        test "$port" -ge 1024 -a "$port" -le 65535 || {
                logger -s "error: port not in range - out: '$output'"
                return 128
        }

        test "${#checksum}" -eq 64 || {
                logger -s "error: checksum != sha256 - out: '$output'"
                return 8
        }

        echo "$port"
}

speedcheck()
{
        local ip="${1:-intercity-vpn.de}"
        local port=

	# check router status down                                                              
	export downB=$(/root/get_bytes_stats.sh -d -p $PASSWORD)  

		port="$( speedcheck_prepare $ip )" 
		if [ "$?" -eq  "0" ]; then
            export resultDown="$( iperf3 --client "$ip" --port "$port" --reverse --json -4 -P 10 -t $STDURATION )"
		fi
        
	# check router status up                                                                                       
	export downBa=$(/root/get_bytes_stats.sh -d -p $PASSWORD)                                                             
	export upB=$(/root/get_bytes_stats.sh -u -p $PASSWORD)  


		port="$( speedcheck_prepare $ip )" 
		if [ "$?" -eq  "0" ]; then
            export resultUp="$( iperf3 --client "$ip" --port "$port" --json -t $STDURATION )"
		fi
        #port="$( speedcheck_prepare $ip )" && {
        #    export resultUp="$( iperf3 --client "$ip" --port "$port" --json )"
        #}

	# check router status up and down                                             
	export upBa=$(/root/get_bytes_stats.sh -u -p $PASSWORD)  
}

# perform downlink measurements
# resultDown=$(iperf3 -c $STSERVER -p $STPORT -P 10 -4 -t $STDURATION -R --json)

# perform uplink measurements
# resultUp=$(iperf3 -c $STSERVER -p $STPORT -P 10 -4 -t $STDURATION --json)

# timestamp for firebase
speedcheck "$STSERVER"
timestamp=$(date +%s)

# check manual parameter
if [ "$1" == "MAN" ]; then
  MANUAL="1";
else
  MANUAL="0";
fi

# check valid parameter
if [ "$2" == "0" ]; then
  VALID="0";
else
  VALID="-1";
fi

# save to file
if [ -z "$resultDown" ] && [ -z "$resultUp" ]; then
	echo "Both speedtest results are empty! This should be logged..."
	exit -1
fi
echo "{\"TestData\": {\"ServerID\":\"$STSERVER\", \"Manual\":\"$MANUAL\", \"Duration\":\"$STDURATION\""\
"}, \"ResultDownload\": $resultDown, \"ResultUpload\": $resultUp, "\
"\"RouterData\": {\"StartStatus\":\"$downB\", \"MidStatus\": \"$upB\", "\
"\"EndStatusDown\":\"$downBa\", \"EndStatusUp\":\"$upBa\", \"Valid\":\"$VALID\"}}" > measurements/$timestamp.json

echo $(date)' : Perform Measurement script finished' >> /var/log/charlie.log;
